package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.mapper.SysDicMapper
import com.gxzc.zen.api.sys.model.SysDic
import com.gxzc.zen.api.sys.service.ISysDicService
import com.gxzc.zen.common.contants.CACHEKEYS
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.cache.CacheManager
import org.springframework.cache.annotation.Cacheable
import org.springframework.stereotype.Service

/**
 * <p>
 * 字典表 服务实现类
 * </p>
 *
 * @author NorthLan123
 * @since 2018-02-06
 */
@Service
class SysDicServiceImpl : ServiceImpl<SysDicMapper, SysDic>(), ISysDicService {


    companion object {
        const val CACHE_KEY_ALL = "'dic_all'"
    }

    @Autowired
    private lateinit var cacheManager: CacheManager

    @Cacheable(CACHEKEYS.SYS, key = CACHE_KEY_ALL)
//    @PostConstruct
    override fun getListCacheable(): MutableList<SysDic> {
        return baseMapper.selectByParams(null)
    }

    override fun getList(enable: Boolean?): MutableList<SysDic> {
        return baseMapper.selectByParams(mutableMapOf(
                "enable" to enable!!
        ))
    }

    //    @Cacheable(CACHEKEYS.SYS, key = "'dic_key_' + #key")
    override fun getListByKey(key: String): MutableList<SysDic> {
        val allData = getListCacheable()
        return allData.filter {
            it.enable != null && it.enable!! && it.key == key
        }.toMutableList()
    }

    //    @CacheEvict(CACHEKEYS.SYS, key = "'dic_key_' + #sysDic.key")
    @Suppress("UNCHECKED_CAST")
    override fun modifySysDic(sysDic: SysDic): SysDic {
        baseMapper.updateById(sysDic)
        // 更新缓存
        val cache = cacheManager.getCache(CACHEKEYS.SYS)
        val cachedList: MutableList<SysDic>? = cache[CACHE_KEY_ALL].get() as MutableList<SysDic>
        cachedList?.let {
            it.removeIf { it.id == sysDic.id }
            it.add(sysDic)
            cache.put(CACHE_KEY_ALL, cachedList)
        }

        return sysDic
    }

    override fun getOneByKey(key: String, value: String?, sort: Int?): SysDic? {
        val result = baseMapper.selectByParams(mutableMapOf(
                "key" to key,
                "value" to value!!,
                "sort" to sort!!,
                "enable" to true
        ))
        return if (result.size > 0) {
            result.filter {
                it.sort == sort && it.value == value
            }[0]
        } else {
            null
        }
    }
}
